@@ -103,6 +103,8 @@ static int cnic_arp_send(nic_t *nic, nic_interface_t *nic_iface, int fd,
 	static const uint8_t multicast_mac[] = {
 				0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
 
+	LOG_DEBUG(PFX "%s: host:%d - try getting xmit mutex cnic arp send",
+		  nic->log_name, nic->host_no);
 	rc = pthread_mutex_trylock(&nic->xmit_mutex);
 	if (rc != 0) {
 		LOG_DEBUG(PFX "%s: could not get xmit_mutex", nic->log_name);
@@ -112,6 +114,7 @@ static int cnic_arp_send(nic_t *nic, nic_interface_t *nic_iface, int fd,
 	eth = (*nic->ops->get_tx_pkt) (nic);
 	if (eth == NULL) {
 		LOG_WARN(PFX "%s: couldn't get tx packet", nic->log_name);
+		pthread_mutex_unlock(&nic->xmit_mutex);
 		return -EAGAIN;
 	}
 
@@ -359,9 +362,8 @@ int cnic_handle_ipv4_iscsi_path_req(nic_t *nic, int fd,
 			       &nic_iface->ustack.default_route_addr,
 			       sizeof(dst_addr));
 		} else {
-			arp_retry = MAX_ARP_RETRY;
-			LOG_DEBUG(PFX "%s: no default", nic->log_name);
-			goto done;
+			LOG_DEBUG(PFX "%s: no default route address",
+				  nic->log_name);
 		}
 	}
 	arp_retry = 0;
