@@ -1014,8 +1014,8 @@ int idbm_rec_update_param(recinfo_t *info, char *name, char *value,
 	int i;
 	int passwd_done = 0;
 	char passwd_len[8];
-	char *tmp_value, *token;
-	bool *found;
+	char *tmp_value, *token, *tmp;
+	bool *found = NULL;
 	int *tmp_data;
 
 setup_passwd_len:
@@ -1079,12 +1079,25 @@ setup_passwd_len:
 				if (!info[i].data)
 					continue;
 				tbl = (void *)info[i].opts[0];
-				/* strsep is destructive, make a copy to work with */
+				/*
+				 * strsep is destructive, make a copy to work with
+				 * tmp_value would be modified in strsep() too, so
+				 * here make a copy of tmp_value to tmp
+				 */
 				tmp_value = strdup(value);
+				if (!tmp_value)
+					return ISCSI_ERR_NOMEM;
+				tmp = tmp_value;
+
 				k = 0;
 				tmp_data = malloc(info[i].data_len);
+				if (!tmp_data)
+					goto free_tmp;
 				memset(tmp_data, ~0, info[i].data_len);
+
 				found = calloc(info[i].numopts, sizeof(bool));
+				if (!found)
+					goto free_tmp_data;
 
 next_token:			while ((token = strsep(&tmp_value, ", \n"))) {
 					if (!strlen(token))
@@ -1113,7 +1126,7 @@ next_token:			while ((token = strsep(&tmp_value, ", \n"))) {
 					            " for '%s'", token, info[i].name);
 				}
 				memcpy(info[i].data, tmp_data, info[i].data_len);
-				free(tmp_value);
+				free(tmp);
 				free(tmp_data);
 				tmp_value = NULL;
 				tmp_data = NULL;
@@ -1135,8 +1148,17 @@ next_token:			while ((token = strsep(&tmp_value, ", \n"))) {
 
 	return ISCSI_ERR_INVAL;
 
+free_tmp_data:
+	free(tmp_data);
+
+free_tmp:
+	free(tmp);
+	return ISCSI_ERR_NOMEM;
+
 updated:
 	strlcpy((char*)info[i].value, value, VALUE_MAXVAL);
+	if (found)
+		free(found);
 
 #define check_password_param(_param) \
 	if (!passwd_done && !strcmp(#_param, name)) { \
@@ -1433,7 +1455,7 @@ int idbm_lock(void)
 	}
 
 	if (access(LOCK_DIR, F_OK) != 0) {
-		if (mkdir(LOCK_DIR, 0660) != 0) {
+		if (mkdir(LOCK_DIR, 0770) != 0) {
 			log_error("Could not open %s: %s", LOCK_DIR,
 				  strerror(errno));
 			return ISCSI_ERR_IDBM;
@@ -2115,7 +2137,7 @@ static FILE *idbm_open_rec_w(char *portal, char *config)
 		}
 
 mkdir_portal:
-		if (mkdir(portal, 0660) != 0) {
+		if (mkdir(portal, 0770) != 0) {
 			log_error("Could not make dir %s err %d",
 				  portal, errno);
 			return NULL;
@@ -2150,7 +2172,7 @@ static int idbm_rec_write(node_rec_t *rec, bool disable_lock)
 
 	snprintf(portal, PATH_MAX, "%s", NODE_CONFIG_DIR);
 	if (access(portal, F_OK) != 0) {
-		if (mkdir(portal, 0660) != 0) {
+		if (mkdir(portal, 0770) != 0) {
 			log_error("Could not make %s: %s", portal,
 				  strerror(errno));
 			rc = ISCSI_ERR_IDBM;
@@ -2160,7 +2182,7 @@ static int idbm_rec_write(node_rec_t *rec, bool disable_lock)
 
 	snprintf(portal, PATH_MAX, "%s/%s", NODE_CONFIG_DIR, rec->name);
 	if (access(portal, F_OK) != 0) {
-		if (mkdir(portal, 0660) != 0) {
+		if (mkdir(portal, 0770) != 0) {
 			log_error("Could not make %s: %s", portal,
 				  strerror(errno));
 			rc = ISCSI_ERR_IDBM;
@@ -2220,7 +2242,7 @@ mkdir_portal:
 	snprintf(portal, PATH_MAX, "%s/%s/%s,%d,%d", NODE_CONFIG_DIR,
 		 rec->name, rec->conn[0].address, rec->conn[0].port, rec->tpgt);
 	if (stat(portal, &statb)) {
-		if (mkdir(portal, 0660) != 0) {
+		if (mkdir(portal, 0770) != 0) {
 			log_error("Could not make dir %s: %s",
 				  portal, strerror(errno));
 			rc = ISCSI_ERR_IDBM;
@@ -2272,7 +2294,7 @@ idbm_discovery_write(discovery_rec_t *rec)
 	snprintf(portal, PATH_MAX, "%s",
 		 disc_type_to_config_vals[rec->type].config_root);
 	if (access(portal, F_OK) != 0) {
-		if (mkdir(portal, 0660) != 0) {
+		if (mkdir(portal, 0770) != 0) {
 			log_error("Could not make %s: %s", portal,
 				  strerror(errno));
 			rc = ISCSI_ERR_IDBM;
@@ -2331,7 +2353,7 @@ static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 		break;
 	case DISCOVERY_TYPE_FW:
 		if (access(FW_CONFIG_DIR, F_OK) != 0) {
-			if (mkdir(FW_CONFIG_DIR, 0660) != 0) {
+			if (mkdir(FW_CONFIG_DIR, 0770) != 0) {
 				log_error("Could not make %s: %s",
 					  FW_CONFIG_DIR, strerror(errno));
 				rc = ISCSI_ERR_IDBM;
@@ -2345,7 +2367,7 @@ static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 		break;
 	case DISCOVERY_TYPE_STATIC:
 		if (access(STATIC_CONFIG_DIR, F_OK) != 0) {
-			if (mkdir(STATIC_CONFIG_DIR, 0660) != 0) {
+			if (mkdir(STATIC_CONFIG_DIR, 0770) != 0) {
 				log_error("Could not make %s; %s",
 					  STATIC_CONFIG_DIR, strerror(errno));
 				rc = ISCSI_ERR_IDBM;
@@ -2359,7 +2381,7 @@ static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 		break;
 	case DISCOVERY_TYPE_ISNS:
 		if (access(ISNS_CONFIG_DIR, F_OK) != 0) {
-			if (mkdir(ISNS_CONFIG_DIR, 0660) != 0) {
+			if (mkdir(ISNS_CONFIG_DIR, 0770) != 0) {
 				log_error("Could not make %s: %s",
 					  ISNS_CONFIG_DIR, strerror(errno));
 				rc = ISCSI_ERR_IDBM;
@@ -2436,6 +2458,12 @@ int idbm_add_node(node_rec_t *newrec, discovery_rec_t *drec, int overwrite)
 		rc = idbm_delete_node(&rec);
 		if (rc)
 			goto unlock;
+
+		if (drec->type == DISCOVERY_TYPE_FW) {
+			log_debug(8, "setting firmware node 'startup' to 'onboot'");
+			newrec->startup = ISCSI_STARTUP_ONBOOT;
+			newrec->conn[0].startup = ISCSI_STARTUP_ONBOOT;
+		}
 		log_debug(7, "overwriting existing record");
 	} else
 		log_debug(7, "adding new DB record");
@@ -3004,7 +3032,7 @@ int idbm_init(idbm_get_config_file_fn *fn)
 {
 	/* make sure root db dir is there */
 	if (access(ISCSI_CONFIG_ROOT, F_OK) != 0) {
-		if (mkdir(ISCSI_CONFIG_ROOT, 0660) != 0) {
+		if (mkdir(ISCSI_CONFIG_ROOT, 0770) != 0) {
 			log_error("Could not make %s %d", ISCSI_CONFIG_ROOT,
 				   errno);
 			return errno;
