@@ -34,6 +34,7 @@
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/prctl.h>
 #ifndef	NO_SYSTEMD
 #include <systemd/sd-daemon.h>
 #endif
@@ -56,6 +57,10 @@
 #include "iscsid_req.h"
 #include "iscsi_err.h"
 
+#ifndef PR_SET_IO_FLUSHER
+#define PR_SET_IO_FLUSHER 57
+#endif
+
 /* global config info */
 struct iscsi_daemon_config daemon_config;
 struct iscsi_daemon_config *dconfig = &daemon_config;
@@ -63,7 +68,7 @@ struct iscsi_daemon_config *dconfig = &daemon_config;
 static char program_name[] = "iscsid";
 static pid_t log_pid;
 static gid_t gid;
-static int daemonize = 1;
+static bool daemonize = true;
 static int mgmt_ipc_fd;
 static int sessions_to_recover = 0;
 
@@ -381,6 +386,8 @@ int main(int argc, char *argv[])
 	struct sigaction sa_new;
 	int control_fd;
 	pid_t pid;
+	bool pid_file_specified = false;
+	bool no_pid_file_specified = false;
 
 	while ((ch = getopt_long(argc, argv, "c:i:fd:nu:g:p:vh", long_options,
 				 &longindex)) >= 0) {
@@ -392,7 +399,7 @@ int main(int argc, char *argv[])
 			initiatorname_file = optarg;
 			break;
 		case 'f':
-			daemonize = 0;
+			daemonize = false;
 			break;
 		case 'd':
 			log_level = atoi(optarg);
@@ -405,9 +412,11 @@ int main(int argc, char *argv[])
 			break;
 		case 'n':
 			pid_file = NULL;
+			no_pid_file_specified = true;
 			break;
 		case 'p':
 			pid_file = optarg;
+			pid_file_specified = true;
 			break;
 		case 'v':
 			printf("%s version %s\n", program_name,
@@ -422,6 +431,17 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (pid_file_specified) {
+		if (no_pid_file_specified) {
+			fprintf(stderr, "error: Conflicting PID-file options requested\n");
+			usage(1);
+		}
+		if (!daemonize) {
+			fprintf(stderr, "error: PID file specified but unused in foreground mode\n");
+			usage(1);
+		}
+	}
+
 	/* initialize logger */
 	log_pid = log_init(program_name, DEFAULT_AREA_SIZE,
 		      daemonize ? log_do_log_daemon : log_do_log_std, NULL);
@@ -442,7 +462,7 @@ int main(int argc, char *argv[])
 		exit(ISCSI_ERR);
 	}
 
-	umask(0177);
+	umask(0077);
 
 	mgmt_ipc_fd = -1;
 	control_fd = -1;
@@ -601,6 +621,15 @@ int main(int argc, char *argv[])
 		exit(ISCSI_ERR);
 	}
 
+	if (prctl(PR_SET_IO_FLUSHER, 1, 0, 0, 0) == -1) {
+		if (errno == EINVAL) {
+			log_info("prctl could not mark iscsid with the PR_SET_IO_FLUSHER flag, because the feature is not supported in this kernel. Will proceed, but iscsid may hang during session level recovery if memory is low.\n");
+		} else {
+			log_error("prctl could not mark iscsid with the PR_SET_IO_FLUSHER flag due to error %s\n",
+				  strerror(errno));
+		}
+	}
+
 	set_state_to_ready();
 	event_loop(ipc, control_fd, mgmt_ipc_fd);
 
