@@ -62,14 +62,14 @@ iscsi_add_text(struct iscsi_hdr *pdu, char *data, int max_data_length,
 	}
 
 	/* param */
-	strncpy(text, param, param_len);
+	memcpy(text, param, param_len);
 	text += param_len;
 
 	/* separator */
 	*text++ = ISCSI_TEXT_SEPARATOR;
 
 	/* value */
-	strncpy(text, value, value_len);
+	memcpy(text, value, value_len);
 	text += value_len;
 
 	/* NUL */
@@ -399,7 +399,7 @@ get_op_params_text_keys(iscsi_session_t *session, int cid,
 			 * what the target gave us.
 			 */
 			if (!conn_rec->iscsi.MaxXmitDataSegmentLength ||
-			    tgt_max_xmit < conn->max_xmit_dlength)
+			    tgt_max_xmit < (int)conn->max_xmit_dlength)
 				conn->max_xmit_dlength = tgt_max_xmit;
 		}
 		text = value_end;
@@ -524,14 +524,9 @@ get_op_params_text_keys(iscsi_session_t *session, int cid,
 		text = value_end;
 	} else if (iscsi_find_key_value("MaxOutstandingR2T", text, end, &value,
 					 &value_end)) {
-		if (session->type == ISCSI_SESSION_TYPE_NORMAL) {
-			if (strcmp(value, "1")) {
-				log_error("Login negotiation "
-					       "failed, can't accept Max"
-					       "OutstandingR2T %s", value);
-				return LOGIN_NEGOTIATION_FAILED;
-			}
-		} else
+		if (session->type == ISCSI_SESSION_TYPE_NORMAL)
+			session->max_r2t = strtoul(value, NULL, 0);
+		else
 			session->irrelevant_keys_bitmap |=
 						IRRELEVANT_MAXOUTSTANDINGR2T;
 		text = value_end;
@@ -629,7 +624,7 @@ check_security_stage_status(iscsi_session_t *session,
 	case AUTH_STATUS_ERROR:
 	case AUTH_STATUS_FAIL:
 	default:
-		if (acl_get_dbg_status(auth_client, &debug_status) !=
+		if (acl_get_dbg_status(auth_client, &debug_status) ==
 		    AUTH_STATUS_NO_ERROR)
 			log_error("Login authentication failed "
 				       "with target %s, %s",
@@ -810,8 +805,9 @@ add_params_normal_session(iscsi_session_t *session, struct iscsi_hdr *pdu,
 		return 0;
 
 	/* these we must have */
+	sprintf(value, "%d", session->max_r2t);
 	if (!iscsi_add_text(pdu, data, max_data_length,
-			    "MaxOutstandingR2T", "1"))
+			    "MaxOutstandingR2T", value))
 		return 0;
 	if (!iscsi_add_text(pdu, data, max_data_length,
 			    "MaxConnections", "1"))
@@ -1266,6 +1262,17 @@ check_for_authentication(iscsi_session_t *session,
 		goto end;
 	}
 
+	int value_list[AUTH_CHAP_ALG_MAX_COUNT];
+
+	if (acl_set_chap_alg_list(auth_client,
+				acl_init_chap_digests(value_list,
+					session->chap_algs,
+					AUTH_CHAP_ALG_MAX_COUNT),
+				value_list) != AUTH_STATUS_NO_ERROR) {
+		log_error("Couldn't set CHAP algorithm list");
+		goto end;
+	}
+
 	if (acl_set_ip_sec(auth_client, 1) != AUTH_STATUS_NO_ERROR) {
 		log_error("Couldn't set IPSec");
 		goto end;
