@@ -19,7 +19,7 @@
  */
 
 #define  _XOPEN_SOURCE 500
-#define _SVID_SOURCE
+#define _DEFAULT_SOURCE
 #include <ftw.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -115,8 +115,11 @@ static int get_iface_from_device(char *id, struct boot_context *context)
 				break;
 			}
 
-			if (sscanf(dent->d_name, "net:%s", context->iface) != 1)
+			if (sscanf(dent->d_name, "net:%s", context->iface) != 1) {
 				rc = EINVAL;
+				break;
+			}
+
 			rc = 0;
 			break;
 		} else {
@@ -221,6 +224,7 @@ static int fill_nic_context(char *subsys, char *id,
 		      sizeof(context->vlan));
 	sysfs_get_str(id, subsys, "subnet-mask", context->mask,
 		      sizeof(context->mask));
+	sysfs_get_int(id, subsys, "prefix-len", &context->prefix);
 	sysfs_get_str(id, subsys, "gateway", context->gateway,
 		      sizeof(context->gateway));
 	sysfs_get_str(id, subsys, "primary-dns", context->primary_dns,
